function submitForm() {
    console.log('submitForm called');
    const xElements = document.getElementsByName('x');
    const y = document.getElementById('y').value.trim();
    const r = document.getElementById('r').value.trim();

    let xSelected = false;
    for (let i = 0; i < xElements.length; i++) {
        if (xElements[i].checked) {
            xSelected = true;
            break;
        }
    }

    if (!xSelected) {
        alert("Пожалуйста, выберите хотя бы одно значение X.");
        return;
    }

    if (!validateInput(y, r)) {
        alert("Некорректные данные для Y или R.");
        return;
    }

    document.getElementById('point-form').submit();
}

function validateInput(y, r) {
    const yValue = parseFloat(y.replace(',', '.'));
    const rValue = parseFloat(r.replace(',', '.'));

    if (isNaN(yValue) || isNaN(rValue)) return false;
    if (yValue < -3 || yValue > 5) return false; // Include boundaries
    if (rValue < 2 || rValue > 5) return false;  // Include boundaries
    return true;
}

function drawGraph(r) {
    if (isNaN(r) || r < 2 || r > 5) {
        r = 2;
    }

    try {
        const canvas = document.getElementById('canvas');
        const ctx = canvas.getContext('2d');

        // Ensure the canvas is properly initialized
        if (!canvas || !ctx) {
            console.error('Canvas or context not found.');
            return;
        }

        const width = canvas.width;
        const height = canvas.height;
        const centerX = width / 2;
        const centerY = height / 2;
        const scale = (width / 2 - 40) / r;

        // Clear the canvas before drawing
        ctx.clearRect(0, 0, width, height);

        // Draw the area (shaded region)
        ctx.fillStyle = 'rgba(0, 0, 255, 0.3)';

        // Triangle (+Y -X quadrant)
        ctx.beginPath();
        ctx.moveTo(centerX, centerY);
        ctx.lineTo(centerX - r * scale, centerY);
        ctx.lineTo(centerX, centerY - (r / 2) * scale);
        ctx.closePath();
        ctx.fill();

        // Quarter Circle (+Y +X quadrant)
        ctx.beginPath();
        ctx.moveTo(centerX, centerY);
        ctx.arc(centerX, centerY, r * scale, -Math.PI / 2, 0, false);
        ctx.closePath();
        ctx.fill();

        // Rectangle (-Y +X quadrant)
        ctx.fillRect(centerX, centerY, r * scale, (r / 2) * scale);

        // Draw axes
        ctx.strokeStyle = 'black';
        ctx.lineWidth = 1;
        ctx.beginPath();
        // X-axis
        ctx.moveTo(0, centerY);
        ctx.lineTo(width, centerY);
        // Y-axis
        ctx.moveTo(centerX, 0);
        ctx.lineTo(centerX, height);
        ctx.stroke();

        // Draw arrows
        ctx.beginPath();
        // X-axis arrow
        ctx.moveTo(width - 10, centerY - 5);
        ctx.lineTo(width, centerY);
        ctx.lineTo(width - 10, centerY + 5);
        // Y-axis arrow
        ctx.moveTo(centerX - 5, 10);
        ctx.lineTo(centerX, 0);
        ctx.lineTo(centerX + 5, 10);
        ctx.stroke();

        // Draw labels
        ctx.fillStyle = 'black';
        ctx.font = '12px Arial';

        // Labels for X-axis
        ctx.fillText('X', width - 15, centerY - 10);
        ctx.fillText('-R', centerX - r * scale - 10, centerY + 15);
        ctx.fillText('-R/2', centerX - (r / 2) * scale - 15, centerY + 15);
        ctx.fillText('R/2', centerX + (r / 2) * scale - 5, centerY + 15);
        ctx.fillText('R', centerX + r * scale - 5, centerY + 15);

        // Labels for Y-axis
        ctx.fillText('Y', centerX + 10, 15);
        ctx.fillText('R', centerX + 5, centerY - r * scale + 5);
        ctx.fillText('R/2', centerX + 5, centerY - (r / 2) * scale + 5);
        ctx.fillText('-R/2', centerX + 5, centerY + (r / 2) * scale + 5);
        ctx.fillText('-R', centerX + 5, centerY + r * scale + 5);

        // Draw tick marks
        ctx.beginPath();
        // X-axis ticks
        ctx.moveTo(centerX - r * scale, centerY - 5);
        ctx.lineTo(centerX - r * scale, centerY + 5);
        ctx.moveTo(centerX - (r / 2) * scale, centerY - 5);
        ctx.lineTo(centerX - (r / 2) * scale, centerY + 5);
        ctx.moveTo(centerX + (r / 2) * scale, centerY - 5);
        ctx.lineTo(centerX + (r / 2) * scale, centerY + 5);
        ctx.moveTo(centerX + r * scale, centerY - 5);
        ctx.lineTo(centerX + r * scale, centerY + 5);

        // Y-axis ticks
        ctx.moveTo(centerX - 5, centerY - r * scale);
        ctx.lineTo(centerX + 5, centerY - r * scale);
        ctx.moveTo(centerX - 5, centerY - (r / 2) * scale);
        ctx.lineTo(centerX + 5, centerY - (r / 2) * scale);
        ctx.moveTo(centerX - 5, centerY + (r / 2) * scale);
        ctx.lineTo(centerX + 5, centerY + (r / 2) * scale);
        ctx.moveTo(centerX - 5, centerY + r * scale);
        ctx.lineTo(centerX + 5, centerY + r * scale);

        ctx.stroke();
    } catch (e) {
        console.error('Error in drawGraph:', e);
    }
}

function drawPoints(results, currentR) {
    try {
        const canvas = document.getElementById('canvas');
        const ctx = canvas.getContext('2d');

        const width = canvas.width;
        const height = canvas.height;
        const centerX = width / 2;
        const centerY = height / 2;
        const scale = (width / 2 - 40) / currentR;

        results.forEach(result => {
            const x = parseFloat(result.x);
            const y = parseFloat(result.y);
            const pointR = parseFloat(result.r);
            const hit = result.hit;

            if (pointR === 0) return;

            const scaleFactor = scale * (currentR / pointR);

            const canvasX = centerX + x * scaleFactor;
            const canvasY = centerY - y * scaleFactor;

            ctx.beginPath();
            ctx.arc(canvasX, canvasY, 3, 0, 2 * Math.PI);
            ctx.fillStyle = hit ? 'green' : 'red'; // Green if hit, red if miss
            ctx.fill();
        });
    } catch (e) {
        console.error('Error in drawPoints:', e);
    }
}

function updateGraph() {
    try {
        console.log('updateGraph called');
        let rInput = document.getElementById('r').value.trim();
        if (rInput === '') {
            console.log('R is empty, skipping graph update.');
            return;
        }
        let rValue = parseFloat(rInput);
        if (isNaN(rValue) || rValue < 2 || rValue > 5) {
            console.log('Invalid R value, using default R = 2');
            rValue = 2; // Default R value
        }
        drawGraph(rValue);
        if (typeof results !== 'undefined' && results.length > 0) {
            drawPoints(results, rValue);
        }
    } catch (e) {
        console.error('Error in updateGraph:', e);
    }
}

let debounceTimer;
window.onload = function() {
    console.log('window.onload called');
    updateGraph();

    const canvas = document.getElementById('canvas');
    canvas.addEventListener('click', function(event) {
        console.log('Canvas clicked');
        let rValue = parseFloat(document.getElementById('r').value.trim());
        if (isNaN(rValue) || rValue < 2 || rValue > 5) {
            alert("Введите корректное значение R перед выбором точки на графике.");
            return;
        }

        const rect = canvas.getBoundingClientRect();
        const xClick = event.clientX - rect.left;
        const yClick = event.clientY - rect.top;

        const width = canvas.width;
        const height = canvas.height;
        const centerX = width / 2;
        const centerY = height / 2;
        const scale = (width / 2 - 40) / rValue;

        const x = (xClick - centerX) / scale;
        const y = (centerY - yClick) / scale;

        const xRounded = Math.round(x);
        const yRounded = Math.round(y * 1000) / 1000;

        const xElements = document.getElementsByName('x');
        for (let i = 0; i < xElements.length; i++) {
            xElements[i].checked = false;
        }
        let closestX = Math.max(-4, Math.min(4, xRounded));
        for (let i = 0; i < xElements.length; i++) {
            if (parseInt(xElements[i].value) === closestX) {
                xElements[i].checked = true;
                break;
            }
        }

        document.getElementById('y').value = yRounded;
        document.getElementById('r').value = rValue;

        submitForm();
    });

    document.getElementById('r').addEventListener('input', function() {
        clearTimeout(debounceTimer);
        debounceTimer = setTimeout(function() {
            console.log('R input changed');
            updateGraph();
        }, 300);
    });

    document.getElementById('clear-button').addEventListener('click', function() {
        console.log('Clear button clicked');
        fetch(contextPath + '/clear', { method: 'POST', credentials: 'include' })
            .then(() => {
                document.getElementById('results').innerHTML = `
                <tr>
                    <th>X</th>
                    <th>Y</th>
                    <th>R</th>
                    <th>Result</th>
                    <th>Timestamp</th>
                    <th>Execution Time (ms)</th>
                </tr>`;
                results = [];
                updateGraph();
            });
    });
};
